@@ -18,33 +18,32 @@
  */
 package org.apache.myfaces.renderkit.html.ext;
 
+import java.io.IOException;
+import java.util.List;
+import java.util.Set;
+
+import javax.faces.component.UIComponent;
+import javax.faces.component.UISelectBoolean;
+import javax.faces.component.UISelectMany;
+import javax.faces.context.FacesContext;
+import javax.faces.context.ResponseWriter;
+import javax.faces.convert.Converter;
+import javax.faces.convert.ConverterException;
+import javax.faces.model.SelectItem;
+import javax.faces.model.SelectItemGroup;
+
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.apache.myfaces.component.UserRoleUtils;
-import org.apache.myfaces.shared_tomahawk.component.DisplayValueOnlyCapable;
-import org.apache.myfaces.shared_tomahawk.renderkit.JSFAttr;
 import org.apache.myfaces.component.html.ext.HtmlSelectManyCheckbox;
 import org.apache.myfaces.custom.checkbox.HtmlCheckbox;
+import org.apache.myfaces.shared_tomahawk.component.DisplayValueOnlyCapable;
+import org.apache.myfaces.shared_tomahawk.renderkit.JSFAttr;
 import org.apache.myfaces.shared_tomahawk.renderkit.RendererUtils;
 import org.apache.myfaces.shared_tomahawk.renderkit.html.HTML;
 import org.apache.myfaces.shared_tomahawk.renderkit.html.HtmlCheckboxRendererBase;
 import org.apache.myfaces.shared_tomahawk.renderkit.html.HtmlRendererUtils;
 
-import javax.faces.FacesException;
-import javax.faces.component.NamingContainer;
-import javax.faces.component.UIComponent;
-import javax.faces.component.UISelectMany;
-import javax.faces.component.UISelectBoolean;
-import javax.faces.context.FacesContext;
-import javax.faces.context.ResponseWriter;
-import javax.faces.convert.Converter;
-import javax.faces.model.SelectItem;
-import javax.faces.model.SelectItemGroup;
-
-import java.io.IOException;
-import java.util.List;
-import java.util.Set;
-
 
 /**
  * @JSFRenderer
@@ -63,7 +62,7 @@ import java.util.Set;
  *   type = "org.apache.myfaces.Checkbox"
  * 
  * @author Manfred Geiler (latest modification by $Author: lu4242 $)
- * @version $Revision: 1395438 $ $Date: 2012-10-07 21:26:13 -0500 (Sun, 07 Oct 2012) $
+ * @version $Revision: 1393888 $ $Date: 2012-10-03 21:51:42 -0500 (Wed, 03 Oct 2012) $
  */
 public class HtmlCheckboxRenderer
         extends HtmlCheckboxRendererBase
@@ -87,7 +86,7 @@ public class HtmlCheckboxRenderer
         }
         else if (component instanceof DisplayValueOnlyCapable && HtmlRendererUtils.isDisplayValueOnly(component))
         {
-            HtmlRendererUtils.renderDisplayValueOnlyForSelects(context, component);
+            HtmlRendererUtils.renderDisplayValueOnlyForSelects(context, component, true);
         }
         else if (component instanceof UISelectMany)
         {
@@ -111,6 +110,7 @@ public class HtmlCheckboxRenderer
         }
     }
 
+    @Override
     public void renderCheckboxList(FacesContext facesContext,
             UISelectMany selectMany) throws IOException
     {
@@ -276,8 +276,16 @@ public class HtmlCheckboxRenderer
 
             writer.startElement(HTML.LABEL_ELEM, selectMany);
             renderLabelClassIfNecessary(facesContext, selectMany, disabled);
-            renderCheckbox(facesContext, selectMany, itemStrValue, selectItem
-                    .getLabel(), disabled, checked, false);
+            renderCheckbox(facesContext, selectMany, itemStrValue, disabled, checked, false,0);
+            writer.write(HTML.NBSP_ENTITY);
+            if(selectItem.isEscape())
+            {
+                writer.writeText(selectItem.getLabel(), null);
+            }
+            else
+            {
+                writer.write(selectItem.getLabel());
+            }
             writer.endElement(HTML.LABEL_ELEM);
         }
     }
@@ -314,7 +322,14 @@ public class HtmlCheckboxRenderer
         if (pageDirectionLayout)
             writer.startElement(HTML.TR_ELEM, selectMany);
         writer.startElement(HTML.TD_ELEM, selectMany);
-        writer.writeText(selectItemGroup.getLabel(), null);
+        if (selectItemGroup.isEscape())
+        {
+            writer.writeText(selectItemGroup.getLabel(), HTML.LABEL_ATTR);
+        }
+        else
+        {
+            writer.write(selectItemGroup.getLabel());
+        }
         writer.endElement(HTML.TD_ELEM);
         
         if (pageDirectionLayout) {
@@ -425,84 +440,29 @@ public class HtmlCheckboxRenderer
 
         ResponseWriter writer = facesContext.getResponseWriter();
         
-        writer.startElement(HTML.LABEL_ELEM, uiSelectMany);
+        //renderCheckbox(facesContext,
+        //               uiSelectMany,
+        //               itemStrValue,
+        //               selectItem.getLabel(),
+        //               isDisabled(facesContext,uiSelectMany),
+        //               lookupSet.contains(itemStrValue), true);
         
-        renderCheckbox(facesContext,
-                       uiSelectMany,
-                       itemStrValue,
-                       selectItem.getLabel(),
-                       isDisabled(facesContext,uiSelectMany),
-                       lookupSet.contains(itemStrValue), false,
-                       index);
+        String itemId = renderCheckbox(facesContext,
+                uiSelectMany,
+                itemStrValue,
+                isDisabled(facesContext,uiSelectMany),
+                lookupSet.contains(itemStrValue), false, index);
+
+        //Render the
+        // label element after the input
+        boolean componentDisabled = isDisabled(facesContext, uiSelectMany);
+        boolean itemDisabled = selectItem.isDisabled();
+        boolean disabled = (componentDisabled || itemDisabled);
+
+        HtmlRendererUtils.renderLabel(writer, uiSelectMany, itemId, selectItem.getLabel(), disabled);
         
-        writer.endElement(HTML.LABEL_ELEM);
     }
 
-    /**
-     * Renders the input item
-     * @return the 'id' value of the rendered element
-     */
-    protected String renderCheckbox(FacesContext facesContext,
-            UIComponent uiComponent, String value, String label,
-            boolean disabled, boolean checked, boolean renderId, int itemNum) throws IOException {
-        String clientId = uiComponent.getClientId(facesContext);
-
-        String itemId = clientId + NamingContainer.SEPARATOR_CHAR + itemNum;
-
-        ResponseWriter writer = facesContext.getResponseWriter();
-
-        writer.startElement(HTML.INPUT_ELEM, uiComponent);
-
-        if (itemId != null)
-        {
-            writer.writeAttribute(HTML.ID_ATTR, itemId, null);
-        }
-        writer.writeAttribute(HTML.TYPE_ATTR, HTML.INPUT_TYPE_CHECKBOX, null);
-        writer.writeAttribute(HTML.NAME_ATTR, clientId, null);
-        
-        if (renderId) {
-            HtmlRendererUtils.writeIdIfNecessary(writer, uiComponent, facesContext);
-        }
-
-        if (checked) {
-            writer.writeAttribute(HTML.CHECKED_ATTR, org.apache.myfaces.shared_tomahawk.renderkit.html.HTML.CHECKED_ATTR, null);
-        }
-        
-        if (disabled) {
-            writer.writeAttribute(HTML.DISABLED_ATTR, HTML.DISABLED_ATTR, null);
-        }
-
-        if ((value != null) && (value.length() > 0)) {
-            writer.writeAttribute(HTML.VALUE_ATTR, value, null);
-        }
-
-        if (uiComponent instanceof UISelectBoolean)
-        {
-            HtmlRendererUtils.renderHTMLAttributes(writer, uiComponent,
-                HTML.INPUT_PASSTHROUGH_ATTRIBUTES_WITHOUT_DISABLED);
-        }
-        else
-        {
-            //HtmlRendererUtils.renderHTMLAttributes(writer, uiComponent,
-            //    HTML.INPUT_PASSTHROUGH_ATTRIBUTES_WITHOUT_DISABLED_AND_STYLE);
-            HtmlRendererUtils.renderHTMLAttributes(writer, uiComponent, HTML.INPUT_ATTRIBUTES);
-            HtmlRendererUtils.renderHTMLAttributes(writer, uiComponent, HTML.COMMON_PASSTROUGH_ATTRIBUTES_WITHOUT_STYLE);
-            HtmlRendererUtils.renderHTMLAttributes(writer, uiComponent, HTML.COMMON_FIELD_ATTRIBUTES_WITHOUT_DISABLED);
-            HtmlRendererUtils.renderHTMLAttributes(writer, uiComponent, HTML.COMMON_FIELD_EVENT_ATTRIBUTES);
-        }
-        if (isDisabled(facesContext, uiComponent)) {
-            writer.writeAttribute(HTML.DISABLED_ATTR, Boolean.TRUE, null);
-        }
-        
-        writer.endElement(HTML.INPUT_ELEM);
-
-        if ((label != null) && (label.length() > 0)) {
-            writer.write(HTML.NBSP_ENTITY);
-            writer.writeText(label, null);
-        }
-
-        return itemId;
-    }
 
     protected boolean isDisabled(FacesContext facesContext, UIComponent uiComponent)
     {
@@ -542,22 +502,47 @@ public class HtmlCheckboxRenderer
         return itemStrValue;
     }
 
-    protected Converter getConverter(FacesContext facesContext,
-            UISelectMany selectMany)
+    /**
+     * Overrides HtmlCheckboxRendererBase to handle valueType attribute on UISelectMany.
+     */
+    @Override
+    public Object getConvertedValue(FacesContext facesContext,
+            UIComponent component, Object submittedValue)
+            throws ConverterException
     {
-        Converter converter;
-        try
+        RendererUtils.checkParamValidity(facesContext, component, null);
+        
+        if (component instanceof UISelectMany) 
         {
-            converter = RendererUtils.findUISelectManyConverter(facesContext,
-                    selectMany);
-        }
-        catch (FacesException e)
+            // invoke getConvertedUISelectManyValue() with considerValueType = true
+            return RendererUtils.getConvertedUISelectManyValue(facesContext,
+                    (UISelectMany) component, submittedValue, true); 
+        } 
+        else 
         {
-            log.error("Error finding Converter for component with id "
-                    + selectMany.getClientId(facesContext));
-            converter = null;
+            // component is not a UISelectMany --> no change needed
+            return super.getConvertedValue(facesContext, component, submittedValue);
+        }
+    }
+    
+    /**
+     * Overrides HtmlCheckboxRendererBase to handle valueType attribute on UISelectMany.
+     */
+    @Override
+    protected Converter getConverter(FacesContext facesContext,
+            UIComponent component)
+    {
+        if (component instanceof UISelectMany)
+        {
+            // invoke findUISelectManyConverterFailsafe() with considerValueType = true
+            return HtmlRendererUtils.findUISelectManyConverterFailsafe(facesContext, 
+                    (UISelectMany) component, true);
+        }
+        else
+        {
+            // component is not a UISelectMany --> no change needed
+            return super.getConverter(facesContext, component);
         }
-        return converter;
     }
 
 }
