@@ -1,48 +1,51 @@
 /*
- *  Licensed to the Apache Software Foundation (ASF) under one
- *  or more contributor license agreements.  See the NOTICE file
- *  distributed with this work for additional information
- *  regarding copyright ownership.  The ASF licenses this file
- *  to you under the Apache License, Version 2.0 (the
- *  "License"); you may not use this file except in compliance
- *  with the License.  You may obtain a copy of the License at
- * 
- *  http://www.apache.org/licenses/LICENSE-2.0
- * 
- *  Unless required by applicable law or agreed to in writing,
- *  software distributed under the License is distributed on an
- *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
- *  KIND, either express or implied.  See the License for the
- *  specific language governing permissions and limitations
- *  under the License.
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
+ *
+ *   http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
  */
 package org.apache.myfaces.shared_tomahawk.renderkit.html;
 
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
-import org.apache.myfaces.shared_tomahawk.renderkit.JSFAttr;
-import org.apache.myfaces.shared_tomahawk.renderkit.RendererUtils;
-import org.apache.myfaces.shared_tomahawk.util.ArrayUtils;
-import org.apache.myfaces.shared_tomahawk.util.StringUtils;
+import java.io.IOException;
+import java.util.List;
+import java.util.Map;
+import java.util.logging.Logger;
 
-import javax.el.ValueExpression;
 import javax.faces.component.UIColumn;
 import javax.faces.component.UIComponent;
 import javax.faces.component.UIData;
+import javax.faces.component.behavior.ClientBehavior;
+import javax.faces.component.behavior.ClientBehaviorHolder;
 import javax.faces.component.html.HtmlColumn;
 import javax.faces.component.html.HtmlDataTable;
 import javax.faces.context.FacesContext;
 import javax.faces.context.ResponseWriter;
-import java.io.IOException;
-import java.util.Iterator;
-import java.util.List;
+
+import org.apache.myfaces.shared_tomahawk.renderkit.JSFAttr;
+import org.apache.myfaces.shared_tomahawk.renderkit.RendererUtils;
+import org.apache.myfaces.shared_tomahawk.renderkit.html.util.JavascriptUtils;
+import org.apache.myfaces.shared_tomahawk.renderkit.html.util.ResourceUtils;
+import org.apache.myfaces.shared_tomahawk.util.ArrayUtils;
+import org.apache.myfaces.shared_tomahawk.util.StringUtils;
 
 /**
  * Common methods for renderers for components that subclass the standard
  * JSF HtmlDataTable component.
  * 
  * @author Thomas Spiegl (latest modification by $Author: lu4242 $)
- * @version $Revision: 1307198 $ $Date: 2012-03-29 20:12:31 -0500 (Thu, 29 Mar 2012) $
+ * @version $Revision: 1307561 $ $Date: 2012-03-30 13:04:07 -0500 (Fri, 30 Mar 2012) $
  */
 public class HtmlTableRendererBase extends HtmlRenderer
 {
@@ -55,13 +58,17 @@ public class HtmlTableRendererBase extends HtmlRenderer
     protected static final String CAPTION_FACET_NAME = "caption";
     
     /** The logger. */
-    private static final Log log = LogFactory.getLog(HtmlTableRendererBase.class);
+    //private static final Log log = LogFactory.getLog(HtmlTableRendererBase.class);
+    private static final Logger log = Logger.getLogger(HtmlTableRendererBase.class.getName());
+    
+    private static final Integer[] ZERO_INT_ARRAY = new Integer[]{0};
 
     /**
      * @param component dataTable
      * @return number of layout columns
      */
-    protected int getNewspaperColumns(UIComponent component) {
+    protected int getNewspaperColumns(UIComponent component)
+    {
         return 1;
     }
 
@@ -69,7 +76,8 @@ public class HtmlTableRendererBase extends HtmlRenderer
      * @param component dataTable
      * @return component to display between layout columns
      */
-    protected UIComponent getNewspaperTableSpacer(UIComponent component) {
+    protected UIComponent getNewspaperTableSpacer(UIComponent component)
+    {
         return null;
     }
 
@@ -77,7 +85,8 @@ public class HtmlTableRendererBase extends HtmlRenderer
      * @param component dataTable
      * @return whether dataTable has component to display between layout columns
      */
-    protected boolean hasNewspaperTableSpacer(UIComponent component) {
+    protected boolean hasNewspaperTableSpacer(UIComponent component)
+    {
         return false;
     }
 
@@ -85,7 +94,8 @@ public class HtmlTableRendererBase extends HtmlRenderer
      * @param component dataTable
      * @return whether dataTable has newspaper columns layed out horizontally
      */
-    protected boolean isNewspaperHorizontalOrientation(UIComponent component) {
+    protected boolean isNewspaperHorizontalOrientation(UIComponent component)
+    {
         return false;
     }
 
@@ -106,6 +116,16 @@ public class HtmlTableRendererBase extends HtmlRenderer
     {
         RendererUtils.checkParamValidity(facesContext, uiComponent, UIData.class);
 
+        Map<String, List<ClientBehavior>> behaviors = null;
+        if (uiComponent instanceof ClientBehaviorHolder)
+        {
+            behaviors = ((ClientBehaviorHolder) uiComponent).getClientBehaviors();
+            if (!behaviors.isEmpty())
+            {
+                ResourceUtils.renderDefaultJsfJsInlineIfNecessary(facesContext, facesContext.getResponseWriter());
+            }
+        }
+        
         beforeTable(facesContext, (UIData) uiComponent);
 
         startTable(facesContext, uiComponent);
@@ -119,8 +139,65 @@ public class HtmlTableRendererBase extends HtmlRenderer
         ResponseWriter writer = facesContext.getResponseWriter();
         HtmlRendererUtils.writePrettyLineSeparator(facesContext);
         writer.startElement(HTML.TABLE_ELEM, uiComponent);
-        HtmlRendererUtils.writeIdIfNecessary(writer, uiComponent, facesContext);
-        HtmlRendererUtils.renderHTMLAttributes(writer, uiComponent, org.apache.myfaces.shared_tomahawk.renderkit.html.HTML.TABLE_PASSTHROUGH_ATTRIBUTES);
+        
+        Map<String, List<ClientBehavior>> behaviors = null;
+        if (uiComponent instanceof ClientBehaviorHolder && JavascriptUtils.isJavascriptAllowed(
+                facesContext.getExternalContext()))
+        {
+            behaviors = ((ClientBehaviorHolder) uiComponent).getClientBehaviors();
+            if (!behaviors.isEmpty())
+            {
+                HtmlRendererUtils.writeIdAndName(writer, uiComponent, facesContext);
+            }
+            else
+            {
+                HtmlRendererUtils.writeIdIfNecessary(writer, uiComponent, facesContext);
+            }
+            if (behaviors.isEmpty() && isCommonPropertiesOptimizationEnabled(facesContext))
+            {
+                CommonPropertyUtils.renderEventProperties(writer, 
+                        CommonPropertyUtils.getCommonPropertiesMarked(uiComponent), uiComponent);
+            }
+            else
+            {
+                if (isCommonEventsOptimizationEnabled(facesContext))
+                {
+                    CommonEventUtils.renderBehaviorizedEventHandlers(facesContext, writer, 
+                           CommonPropertyUtils.getCommonPropertiesMarked(uiComponent),
+                           CommonEventUtils.getCommonEventsMarked(uiComponent), uiComponent, behaviors);
+                }
+                else
+                {
+                    HtmlRendererUtils.renderBehaviorizedEventHandlers(facesContext, writer, uiComponent, behaviors);
+                }
+            }
+            if (isCommonPropertiesOptimizationEnabled(facesContext))
+            {
+                HtmlRendererUtils.renderHTMLAttributes(writer, uiComponent, HTML.TABLE_ATTRIBUTES);
+                CommonPropertyUtils.renderCommonPassthroughPropertiesWithoutEvents(writer, 
+                        CommonPropertyUtils.getCommonPropertiesMarked(uiComponent), uiComponent);
+            }
+            else
+            {
+                HtmlRendererUtils.renderHTMLAttributes(writer, uiComponent, 
+                        HTML.TABLE_PASSTHROUGH_ATTRIBUTES_WITHOUT_EVENTS);
+            }
+        }
+        else
+        {
+            HtmlRendererUtils.writeIdIfNecessary(writer, uiComponent, facesContext);
+            if (isCommonPropertiesOptimizationEnabled(facesContext))
+            {
+                HtmlRendererUtils.renderHTMLAttributes(writer, uiComponent, HTML.TABLE_ATTRIBUTES);
+                CommonPropertyUtils.renderCommonPassthroughProperties(writer, 
+                        CommonPropertyUtils.getCommonPropertiesMarked(uiComponent), uiComponent);
+            }
+            else
+            {
+                HtmlRendererUtils.renderHTMLAttributes(writer, uiComponent, 
+                        HTML.TABLE_PASSTHROUGH_ATTRIBUTES);
+            }
+        }
     }
 
     /**
@@ -132,18 +209,10 @@ public class HtmlTableRendererBase extends HtmlRenderer
     {
         RendererUtils.checkParamValidity(facesContext, component, UIData.class);
 
-        ResponseWriter writer = facesContext.getResponseWriter();
-
         beforeBody(facesContext, (UIData) component);
 
-        HtmlRendererUtils.writePrettyLineSeparator(facesContext);
-        writer.startElement(HTML.TBODY_ELEM, component);
-        writer.writeAttribute(HTML.ID_ATTR, component.getClientId(facesContext)+":tbody_element", null);
-
         encodeInnerHtml(facesContext, component);
 
-        writer.endElement(HTML.TBODY_ELEM);
-
         afterBody(facesContext, (UIData) component);
     }
 
@@ -158,18 +227,45 @@ public class HtmlTableRendererBase extends HtmlRenderer
             throws IOException
     {
         HtmlRendererUtils.renderTableCaption(facesContext, writer, component);
-    }    
+    }  
+    
+    /**
+     * Renders the colgroups facet.
+     * @param facesContext the <code>FacesContext</code>.
+     * @param writer the <code>ResponseWriter</code>.
+     * @param component the parent <code>UIComponent</code> containing the facets.
+     * @throws IOException if an exception occurs.
+     * @since 2.0
+     */
+    protected void renderColgroupsFacet(FacesContext facesContext, ResponseWriter writer, UIComponent component)
+            throws IOException
+    {
+        UIComponent colgroupsFacet = component.getFacet("colgroups");
+        if (colgroupsFacet == null)
+        {
+            // no facet to be rendered
+            return;
+        }
+        // render the facet
+        HtmlRendererUtils.writePrettyLineSeparator(facesContext);
+        //RendererUtils.renderChild(facesContext, colgroupsFacet);
+        colgroupsFacet.encodeAll(facesContext);
+    } 
     
     /**
      * Gets styles for the specified component.
      */
-    protected static Styles getStyles(UIData uiData) {
+    protected static Styles getStyles(UIData uiData)
+    {
         String rowClasses;
         String columnClasses;
-        if(uiData instanceof HtmlDataTable) {
+        if(uiData instanceof HtmlDataTable) 
+        {
             rowClasses = ((HtmlDataTable)uiData).getRowClasses();
             columnClasses = ((HtmlDataTable)uiData).getColumnClasses();
-        } else {
+        }
+        else
+        {
             rowClasses = (String)uiData.getAttributes().get(JSFAttr.ROW_CLASSES_ATTR);
             columnClasses = (String)uiData.getAttributes().get(JSFAttr.COLUMN_CLASSES_ATTR);
         }
@@ -179,12 +275,14 @@ public class HtmlTableRendererBase extends HtmlRenderer
     /**
      * Class manages the styles from String lists.
      */
-    protected static class Styles {
+    protected static class Styles
+    {
 
         private String[] _columnStyle;
         private String[] _rowStyle;
 
-        Styles(String rowStyles, String columnStyles) {
+        Styles(String rowStyles, String columnStyles)
+        {
             _rowStyle = (rowStyles == null)
                 ? ArrayUtils.EMPTY_STRING_ARRAY
                 : StringUtils.trim(
@@ -195,29 +293,62 @@ public class HtmlTableRendererBase extends HtmlRenderer
                     StringUtils.splitShortString(columnStyles, ','));
         }
 
-        public String getRowStyle(int idx) {
-            if(!hasRowStyle()) {
+        public String getRowStyle(int idx)
+        {
+            if(!hasRowStyle())
+            {
                 return null;
             }
             return _rowStyle[idx % _rowStyle.length];
         }
 
-        public String getColumnStyle(int idx) {
-            if(!hasColumnStyle()) {
+        public String getColumnStyle(int idx)
+        {
+            if(!hasColumnStyle())
+            {
                 return null;
             }
-            return _columnStyle[idx % _columnStyle.length];
+            //return _columnStyle[idx % _columnStyle.length];
+            if (idx < _columnStyle.length)
+            {
+                return _columnStyle[idx];
+            }
+            return null;   
         }
 
-        public boolean hasRowStyle() {
+        public boolean hasRowStyle()
+        {
             return _rowStyle.length > 0;
         }
 
-        public boolean hasColumnStyle() {
+        public boolean hasColumnStyle()
+        {
             return _columnStyle.length > 0;
         }
     }
 
+    private Integer[] getBodyRows(FacesContext facesContext, UIComponent component)
+    {
+        Integer[] bodyrows = null;
+        String bodyrowsAttr = (String) component.getAttributes().get(JSFAttr.BODYROWS_ATTR);
+        if(bodyrowsAttr != null && !"".equals(bodyrowsAttr)) 
+        {   
+            String[] bodyrowsString = StringUtils.trim(StringUtils.splitShortString(bodyrowsAttr, ','));
+            // parsing with no exception handling, because of JSF-spec: 
+            // "If present, this must be a comma separated list of integers."
+            bodyrows = new Integer[bodyrowsString.length];
+            for(int i = 0; i < bodyrowsString.length; i++) 
+            {
+                bodyrows[i] = new Integer(bodyrowsString[i]);
+            }
+            
+        }
+        else
+        {
+            bodyrows = ZERO_INT_ARRAY;
+        }
+        return bodyrows;
+    }
 
     /**
      * Renders everything inside the TBODY tag by iterating over the row objects
@@ -227,8 +358,8 @@ public class HtmlTableRendererBase extends HtmlRenderer
      * This method is separated from the encodeChildren so that it can be overridden by
      * subclasses. One class that uses this functionality is autoUpdateDataTable.
      */
-     public void encodeInnerHtml(FacesContext facesContext, UIComponent component)throws IOException{
-
+     public void encodeInnerHtml(FacesContext facesContext, UIComponent component)throws IOException
+     {
         UIData uiData = (UIData) component;
         ResponseWriter writer = facesContext.getResponseWriter();
 
@@ -245,10 +376,13 @@ public class HtmlTableRendererBase extends HtmlRenderer
         if (rowCount == 0)
         {
             //nothing to render, to get valid xhtml we render an empty dummy row
+            writer.startElement(HTML.TBODY_ELEM, uiData);
+            writer.writeAttribute(HTML.ID_ATTR, component.getClientId(facesContext) + ":tbody_element", null);
             writer.startElement(HTML.TR_ELEM, uiData);
             writer.startElement(HTML.TD_ELEM, uiData);
             writer.endElement(HTML.TD_ELEM);
             writer.endElement(HTML.TR_ELEM);
+            writer.endElement(HTML.TBODY_ELEM);
             return;
         }
 
@@ -283,6 +417,10 @@ public class HtmlTableRendererBase extends HtmlRenderer
             newspaperRows = ((last - first) / newspaperColumns) + 1;
         }
         boolean newspaperHorizontalOrientation = isNewspaperHorizontalOrientation(component);
+        
+        // get the row indizes for which a new TBODY element should be created
+        Integer[] bodyrows = getBodyRows(facesContext, component);
+        int bodyrowsCount = 0;
 
         // walk through the newspaper rows
         for(int nr = 0; nr < newspaperRows; nr++)
@@ -313,7 +451,7 @@ public class HtmlTableRendererBase extends HtmlRenderer
                 uiData.setRowIndex(currentRow);
                 if(!uiData.isRowAvailable())
                 {
-                    log.error("Row is not available. Rowindex = " + currentRow);
+                    log.severe("Row is not available. Rowindex = " + currentRow);
                     break;
                 }
     
@@ -322,6 +460,23 @@ public class HtmlTableRendererBase extends HtmlRenderer
                     // first column in table, start new row
                     beforeRow(facesContext, uiData);
 
+                    // is the current row listed in the bodyrows attribute
+                    if(ArrayUtils.contains(bodyrows, currentRow))  
+                    {
+                        // close any preopened TBODY element first
+                        if(bodyrowsCount != 0) 
+                        {
+                            HtmlRendererUtils.writePrettyLineSeparator(facesContext);
+                            writer.endElement(HTML.TBODY_ELEM);
+                        }
+                        HtmlRendererUtils.writePrettyLineSeparator(facesContext);
+                        writer.startElement(HTML.TBODY_ELEM, uiData); 
+                        // Do not attach bodyrowsCount to the first TBODY element, because of backward compatibility
+                        writer.writeAttribute(HTML.ID_ATTR, component.getClientId(facesContext) + ":tbody_element" + 
+                            (bodyrowsCount == 0 ? "" : bodyrowsCount), null);
+                        bodyrowsCount++;
+                    }
+                    
                     HtmlRendererUtils.writePrettyLineSeparator(facesContext);
                     renderRowStart(facesContext, writer, uiData, styles, nr);
                     rowStartRendered = true;
@@ -369,8 +524,15 @@ public class HtmlTableRendererBase extends HtmlRenderer
                 afterRow(facesContext, uiData);
             }
         }
+        
+        if(bodyrowsCount != 0)
+        {
+            // close the last TBODY element
+            HtmlRendererUtils.writePrettyLineSeparator(facesContext);
+            writer.endElement(HTML.TBODY_ELEM);
+        }
     }
-
+     
     private void encodeInnerHtmlUnknownRowCount(FacesContext facesContext, UIComponent component)throws IOException
     {
         UIData uiData = (UIData) component;
@@ -378,6 +540,9 @@ public class HtmlTableRendererBase extends HtmlRenderer
 
         Styles styles = getStyles(uiData);
         
+        Integer[] bodyrows = getBodyRows(facesContext, component);
+        int bodyrowsCount = 0;
+        
         int first = uiData.getFirst();
         int rows = uiData.getRows();
         int currentRow = first;
@@ -396,6 +561,23 @@ public class HtmlTableRendererBase extends HtmlRenderer
             // first column in table, start new row
             beforeRow(facesContext, uiData);
 
+            // is the current row listed in the bodyrows attribute
+            if(ArrayUtils.contains(bodyrows, currentRow))  
+            {
+                // close any preopened TBODY element first
+                if(bodyrowsCount != 0) 
+                {
+                    HtmlRendererUtils.writePrettyLineSeparator(facesContext);
+                    writer.endElement(HTML.TBODY_ELEM);
+                }
+                HtmlRendererUtils.writePrettyLineSeparator(facesContext);
+                writer.startElement(HTML.TBODY_ELEM, uiData); 
+                // Do not attach bodyrowsCount to the first TBODY element, because of backward compatibility
+                writer.writeAttribute(HTML.ID_ATTR, component.getClientId(facesContext) + ":tbody_element" + 
+                    (bodyrowsCount == 0 ? "" : bodyrowsCount), null);
+                bodyrowsCount++;
+            }
+            
             HtmlRendererUtils.writePrettyLineSeparator(facesContext);
             renderRowStart(facesContext, writer, uiData, styles, currentRow);
             
@@ -440,12 +622,22 @@ public class HtmlTableRendererBase extends HtmlRenderer
         if (!isRowRendered)
         {
             //nothing to render, to get valid xhtml we render an empty dummy row
+            writer.startElement(HTML.TBODY_ELEM, uiData);
+            writer.writeAttribute(HTML.ID_ATTR, component.getClientId(facesContext) + ":tbody_element", null);
             writer.startElement(HTML.TR_ELEM, uiData);
             writer.startElement(HTML.TD_ELEM, uiData);
             writer.endElement(HTML.TD_ELEM);
             writer.endElement(HTML.TR_ELEM);
+            writer.endElement(HTML.TBODY_ELEM);
             return;
         }
+
+        if(bodyrowsCount != 0)
+        {
+            // close the last TBODY element
+            HtmlRendererUtils.writePrettyLineSeparator(facesContext);
+            writer.endElement(HTML.TBODY_ELEM);
+        }
     }
 
     protected void encodeColumnChild(FacesContext facesContext, ResponseWriter writer,
@@ -476,14 +668,33 @@ public class HtmlTableRendererBase extends HtmlRenderer
             UIComponent component,
             Styles styles, int columnStyleIndex) throws IOException
     {
-        writer.startElement(HTML.TD_ELEM, uiData);
+        // Get the rowHeader attribute from the attribute map, because of MYFACES-1790
+        Object rowHeaderAttr = component.getAttributes().get(JSFAttr.ROW_HEADER_ATTR);
+        boolean rowHeader = rowHeaderAttr != null && ((Boolean) rowHeaderAttr);
+        
+        if(rowHeader) 
+        {
+            writer.startElement(HTML.TH_ELEM, uiData);   
+            writer.writeAttribute(HTML.SCOPE_ATTR, HTML.SCOPE_ROW_VALUE, null);
+        }
+        else 
+        {
+            writer.startElement(HTML.TD_ELEM, uiData);
+        }
         if (styles.hasColumnStyle())
         {
             writer.writeAttribute(HTML.CLASS_ATTR, styles.getColumnStyle(columnStyleIndex), null);
         }
-    
-        RendererUtils.renderChild(facesContext, component);
-        writer.endElement(HTML.TD_ELEM);
+        //RendererUtils.renderChild(facesContext, component);
+        component.encodeAll(facesContext);
+        if(rowHeader) 
+        {
+            writer.endElement(HTML.TH_ELEM);   
+        }
+        else 
+        {
+            writer.endElement(HTML.TD_ELEM);
+        }
     }
 
     /**
@@ -511,9 +722,11 @@ public class HtmlTableRendererBase extends HtmlRenderer
         }
     }
 
-    protected void renderRowStyle(FacesContext facesContext, ResponseWriter writer, UIData uiData, Styles styles, int rowStyleIndex) throws IOException
+    protected void renderRowStyle(FacesContext facesContext, ResponseWriter writer, 
+            UIData uiData, Styles styles, int rowStyleIndex) throws IOException
     {
-        if(styles.hasRowStyle()) {
+        if(styles.hasRowStyle())
+        {
             String rowStyle = styles.getRowStyle(rowStyleIndex);
             writer.writeAttribute(HTML.CLASS_ATTR, rowStyle, null);
         }
@@ -561,6 +774,7 @@ public class HtmlTableRendererBase extends HtmlRenderer
         ResponseWriter writer = facesContext.getResponseWriter();
 
         renderCaptionFacet(facesContext, writer, uiData);
+        renderColgroupsFacet(facesContext, writer, uiData);
         renderFacet(facesContext, writer, uiData, true);
         renderFacet(facesContext, writer, uiData, false);
     }
@@ -614,7 +828,8 @@ public class HtmlTableRendererBase extends HtmlRenderer
      * @param header true if the header of the column child is rendered
      * @param columnIndex the index of the currenly rendered column
      */
-    protected void beforeColumnHeaderOrFooter(FacesContext facesContext, UIData uiData, boolean header, int columnIndex) throws IOException
+    protected void beforeColumnHeaderOrFooter(FacesContext facesContext, UIData uiData, boolean header,
+            int columnIndex) throws IOException
     {         
     }
     /**
@@ -625,7 +840,8 @@ public class HtmlTableRendererBase extends HtmlRenderer
      * @param header true if the header of the column child is rendered
      * @param columnIndex the index of the currenly rendered column
      */
-    protected void afterColumnHeaderOrFooter(FacesContext facesContext, UIData uiData, boolean header, int columnIndex) throws IOException
+    protected void afterColumnHeaderOrFooter(FacesContext facesContext, UIData uiData, boolean header,
+            int columnIndex) throws IOException
     {         
     }
 
@@ -708,9 +924,10 @@ public class HtmlTableRendererBase extends HtmlRenderer
     {
         int colspan = 0;
         boolean hasColumnFacet = false;
-        for (Iterator it = getChildren(component).iterator(); it.hasNext();)
+        int childCount = component.getChildCount();
+        for (int i = 0; i < childCount; i++)
         {
-            UIComponent uiComponent = (UIComponent) it.next();
+            UIComponent uiComponent = component.getChildren().get(i);
             if(uiComponent.isRendered())
             {
                 // a UIColumn has a span of 1, anything else has a span of 0
@@ -720,13 +937,18 @@ public class HtmlTableRendererBase extends HtmlRenderer
                 // the specified type.
                 if (!hasColumnFacet)
                 {
-                     hasColumnFacet = hasFacet(header, uiComponent);
+                    hasColumnFacet = hasFacet(header, uiComponent);
                 }
             }
         }
 
-        UIComponent facet = header ? (UIComponent) component.getFacets().get(HEADER_FACET_NAME)
-                : (UIComponent) component.getFacets().get(FOOTER_FACET_NAME);
+        
+        UIComponent facet = null;
+        if (component.getFacetCount() > 0)
+        {
+            facet = header ? (UIComponent) component.getFacets().get(HEADER_FACET_NAME) 
+                    : (UIComponent) component.getFacets().get(FOOTER_FACET_NAME);
+        }
         if (facet != null || hasColumnFacet)
         {
             // Header or Footer present on either the UIData or a column, so we
@@ -742,17 +964,25 @@ public class HtmlTableRendererBase extends HtmlRenderer
             {
                 String headerStyleClass = getHeaderClass(component);
                 if (facet != null)
+                {
                     renderTableHeaderRow(facesContext, writer, component, facet, headerStyleClass, colspan);
+                }
                 if (hasColumnFacet)
+                {
                     renderColumnHeaderRow(facesContext, writer, component, headerStyleClass);
+                }
             }
             else
             {
                 String footerStyleClass = getFooterClass(component);
                 if (hasColumnFacet)
+                {
                     renderColumnFooterRow(facesContext, writer, component, footerStyleClass);
+                }
                 if (facet != null)
+                {
                     renderTableFooterRow(facesContext, writer, component, facet, footerStyleClass, colspan);
+                }
             }
             if (elemName != null)
             {
@@ -814,7 +1044,8 @@ public class HtmlTableRendererBase extends HtmlRenderer
     protected void renderTableHeaderRow(FacesContext facesContext, ResponseWriter writer, UIComponent component,
             UIComponent headerFacet, String headerStyleClass, int colspan) throws IOException
     {
-        renderTableHeaderOrFooterRow(facesContext, writer, component, headerFacet, headerStyleClass, determineHeaderCellTag(facesContext, component),
+        renderTableHeaderOrFooterRow(facesContext, writer, component, headerFacet, headerStyleClass, 
+                determineHeaderCellTag(facesContext, component),
                 colspan, true);
     }
 
@@ -867,8 +1098,10 @@ public class HtmlTableRendererBase extends HtmlRenderer
         renderColumnHeaderOrFooterRow(facesContext, writer, component, footerStyleClass, false);
     }
 
-    protected void renderTableHeaderOrFooterRow(FacesContext facesContext, ResponseWriter writer, UIComponent component,
-            UIComponent facet, String styleClass, String colElementName, int colspan, boolean isHeader) throws IOException
+    protected void renderTableHeaderOrFooterRow(FacesContext facesContext, ResponseWriter writer, 
+            UIComponent component,
+            UIComponent facet, String styleClass, String colElementName, int colspan, boolean isHeader)
+            throws IOException
     {
         HtmlRendererUtils.writePrettyLineSeparator(facesContext);
         writer.startElement(HTML.TR_ELEM, component);
@@ -896,7 +1129,8 @@ public class HtmlTableRendererBase extends HtmlRenderer
         }
         if (facet != null)
         {
-            RendererUtils.renderChild(facesContext, facet);
+            //RendererUtils.renderChild(facesContext, facet);
+            facet.encodeAll(facesContext);
         }
         writer.endElement(colElementName);
         writer.endElement(HTML.TR_ELEM);
@@ -915,18 +1149,22 @@ public class HtmlTableRendererBase extends HtmlRenderer
         int newspaperColumns = getNewspaperColumns(component);
         for(int nc = 0; nc < newspaperColumns; nc++)
         {
-            for (Iterator it = getChildren(component).iterator(); it.hasNext();)
+            for (int i = 0, childCount = component.getChildCount(); i < childCount; i++)
             {
-                UIComponent uiComponent = (UIComponent) it.next();
+                UIComponent uiComponent = component.getChildren().get(i);
                 if (uiComponent.isRendered())
                 {
                     if (component instanceof UIData && uiComponent instanceof UIColumn)
-                        beforeColumnHeaderOrFooter(facesContext, (UIData)component, header, columnIndex);
+                    {
+                        beforeColumnHeaderOrFooter(facesContext, (UIData) component, header, columnIndex);
+                    }
                 
                     renderColumnChildHeaderOrFooterRow(facesContext, writer, uiComponent, styleClass, header);
                     
                     if (component instanceof UIData && uiComponent instanceof UIColumn)
-                        afterColumnHeaderOrFooter(facesContext, (UIData)component, header, columnIndex);
+                    {
+                        afterColumnHeaderOrFooter(facesContext, (UIData) component, header, columnIndex);
+                    }
                 }
                 columnIndex += 1;
             }
@@ -934,7 +1172,10 @@ public class HtmlTableRendererBase extends HtmlRenderer
             if (hasNewspaperTableSpacer(component))
             {
                 // draw the spacer facet
-                if(nc < newspaperColumns - 1) renderSpacerCell(facesContext, writer, component);
+                if(nc < newspaperColumns - 1)
+                {
+                    renderSpacerCell(facesContext, writer, component);
+                }
             }
         }
         writer.endElement(HTML.TR_ELEM);
@@ -943,12 +1184,18 @@ public class HtmlTableRendererBase extends HtmlRenderer
       /**
       * Renders a spacer between adjacent newspaper columns.
       */
-    protected void renderSpacerCell(FacesContext facesContext, ResponseWriter writer, UIComponent component) throws IOException {
+    protected void renderSpacerCell(FacesContext facesContext, ResponseWriter writer, UIComponent component)
+        throws IOException 
+    {
         UIComponent spacer = getNewspaperTableSpacer(component);
-        if(spacer == null) return;
+        if(spacer == null)
+        {
+            return;
+        }
          
          writer.startElement(HTML.TD_ELEM, component);
-         RendererUtils.renderChild(facesContext, spacer);
+         //RendererUtils.renderChild(facesContext, spacer);
+         spacer.encodeAll(facesContext);
          writer.endElement(HTML.TD_ELEM);
      }
 
@@ -958,26 +1205,38 @@ public class HtmlTableRendererBase extends HtmlRenderer
         if (uiComponent instanceof UIColumn)
         {
             // allow column to override style class, new in JSF 1.2
-            if (uiComponent instanceof HtmlColumn) {
+            if (uiComponent instanceof HtmlColumn)
+            {
                 HtmlColumn column = (HtmlColumn)uiComponent;
                 if (isHeader && column.getHeaderClass()!=null)
+                {
                     styleClass = column.getHeaderClass();
+                }
                 else if (!isHeader && column.getFooterClass()!=null)
+                {
                     styleClass = column.getFooterClass();
-            }else{
+                }
+            }
+            else
+            {
                 //This code corrects MYFACES-1790, because HtmlColumnTag
                 //has as component type javax.faces.Column, so as side
                 //effect it not create HtmlColumn, it create UIColumn
                 //classes.
                 UIColumn column = (UIColumn) uiComponent;                
-                if (isHeader){
+                if (isHeader)
+                {
                     String headerClass = (String) column.getAttributes().get("headerClass");
-                    if (headerClass != null){
+                    if (headerClass != null)
+                    {
                         styleClass = (String) headerClass;
                     }
-                }else{
+                }
+                else
+                {
                     String footerClass = (String) column.getAttributes().get("footerClass");
-                    if (footerClass != null){
+                    if (footerClass != null)
+                    {
                         styleClass = (String) footerClass;
                     }
                 }
@@ -1044,7 +1303,8 @@ public class HtmlTableRendererBase extends HtmlRenderer
 
         if (facet != null)
         {
-            RendererUtils.renderChild(facesContext, facet);
+            //RendererUtils.renderChild(facesContext, facet);
+            facet.encodeAll(facesContext);
         }
         writer.endElement(determineHeaderCellTag(facesContext, uiComponent.getParent()));
     }
@@ -1095,7 +1355,8 @@ public class HtmlTableRendererBase extends HtmlRenderer
         }
         if (facet != null)
         {
-            RendererUtils.renderChild(facesContext, facet);
+            //RendererUtils.renderChild(facesContext, facet);
+            facet.encodeAll(facesContext);
         }
         writer.endElement(HTML.TD_ELEM);
     }
@@ -1113,7 +1374,8 @@ public class HtmlTableRendererBase extends HtmlRenderer
         }
         else
         {
-            return (String) component.getAttributes().get(org.apache.myfaces.shared_tomahawk.renderkit.JSFAttr.HEADER_CLASS_ATTR);
+            return (String) component.getAttributes().get(
+                    org.apache.myfaces.shared_tomahawk.renderkit.JSFAttr.HEADER_CLASS_ATTR);
         }
     }
 
@@ -1130,13 +1392,16 @@ public class HtmlTableRendererBase extends HtmlRenderer
         }
         else
         {
-            return (String) component.getAttributes().get(org.apache.myfaces.shared_tomahawk.renderkit.JSFAttr.FOOTER_CLASS_ATTR);
+            return (String) component.getAttributes().get(
+                    org.apache.myfaces.shared_tomahawk.renderkit.JSFAttr.FOOTER_CLASS_ATTR);
         }
     }
 
     public void decode(FacesContext context, UIComponent component)
     {
         super.decode(context, component);
+        
+        HtmlRendererUtils.decodeClientBehaviors(context, component);
     }
 
 }
